'' +--------------------------------------------------------------------------+
'' | Cluso's Propeller Operating System - OS module: _DIRW    --> _DIRW.CMD   |
'' +--------------------------------------------------------------------------+
'' |  Authors:       (c)2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)|
'' |  Modifications: (c)2012 "Cluso99" (Ray Rodrick)                          |
'' |  License:       MIT License - See end of file for terms of use           |
'' +--------------------------------------------------------------------------+
'' This OS module uses an included module to perform most of the housekeeping.
'' Each module is called from, and returns to, the prop binary "_CMD.CMD"
'' Compile with PropTool + "PreSpin 0.3" by Andy Schenk www.insonix.ch/propeller                             
'' Some OS modules are derived from: 
''      KyeDOS v3 (c) 2011 by Kwabena W. Agyman (Kye) & J. Moxham (Dr_Acula)
'' Uses ExpandWildcards and Match code from Sphinx DIR - thanks Michael Park.
'' See the included code for history and acknowledgments.
''
'' RR20120317   This module does......

'' Bugs:  1.  DIR D*.TXT does not work correctly
''        2.  need to get _OS_ROWS and _OS_COLUMNS from hubdef

'#include "__MODULE.spin"        ' include the common code for OS modules 
#include "__MODULE.spin"

DAT
_ModuleStr    byte      "=== DIRW ===",0        'module's name (string)

 
PRI executeCommand(StringPointer) | buffer, line, lines

    stringPointer := str.tokenizeString(0)              ' get the parameters
    str.copy(stringpointer,@LineOfText1)                ' move to lineoftext1
    ExpandWildcards("?")                                ' expands lineoftext1 -> lineoftext2
    str.copy(@LineOfText2, @LineOfText3)                ' save to lineoftext3
    lines := (byte[_hub#_OS_Rows] & _hub#_ROW_MASK) - 2 
    line~
    
    fat.listEntries("W")   
    repeat while(buffer := fat.listEntries("N"))
      str.copy(buffer,@LineOfText1)                     ' copy filename to LineOfText1
      str.TrimString(@LineOfText1)
      ExpandWildcards(" ")                              ' expands into lineoftext2
      if Match                                          ' @LineOfText2==@LineOfText3 length=12
        PrintString(@LineOfText2)                       ' print filename 8.3
        PrintDetails
        crlf
        line++
      if line => lines                                  ' filled screen?
        PrintString(string("more..."))
        GetKey                                          ' wait for any key
        line~
        crlf
    crlf
    abort 0                                             ' so knows a command was run

PRI PrintDetails | i, p
'Print size and modified date/time 
    printstring(string(" "))
    'convert file size into KB
    i := fat.listSize
    if i > 0 and i < 1024
      i := 1024
    i := i / 1024
    str.copy(1 + str.integerToDecimal(i, 7),@LineOfText1)
    i~
    repeat while LineOfText1[i] == "0" and i < 6
      LineOfText1[i++] := " "
    printstring(@LineOfText1)
    printstring(string(" KB  "))
    printstring(1 + str.integerToDecimal(fat.listModificationYear, 4))
    PrintChar("/")
    printstring(1 + str.integerToDecimal(fat.listModificationMonth, 2))
    PrintChar("/")
    printstring(1 + str.integerToDecimal(fat.listModificationDay, 2))
    PrintChar(" ")
    printstring(1 + str.integerToDecimal(fat.listModificationHours, 2))
    PrintChar(":")
    printstring(1 + str.integerToDecimal(fat.listModificationMinutes, 2))

PRI ExpandWildcards(char) | i, j
'Expand the string lineoftext1 -> lineoftext2 using the passed wildcard "?" or " "
'lineoftext2 is set to a string of 12 characters for the FAT filename 8.3 format
  i~
  j~
  repeat while LineOfText1[j]                           ' expand "*" with "?" 
    if LineOfText1[j] == "*"
      if i < 8
        repeat
          LineOfText2[i++] := char                      ' "?"
        until i == 8
      elseif i < 11
        repeat
          LineOfText2[i++] := char                      ' "?"
        until i == 11
      else
        printstring(string("bad wildcard"))
        abort string("bad wildcard")
    elseif LineOfText1[j] == "."
      repeat while i < 8
        LineOfText2[i++] := char                        ' "?"
      if i > 8
        printstring(string("filename too long"))
        abort string("filename too long")
    else
      if i < 11
        LineOfText2[i++] := LineOfText1[j]
      else
        printstring(string("filename too long"))
        abort string("filename too long")
    j++
  repeat while i < 11
    LineOfText2[i++] := char                            ' "?"
  'now add in the "." and string terminator
  LineOfText2[12] := 0                           
  LineOfText2[11] := LineOfText2[10]
  LineOfText2[10] := LineOfText2[9]
  LineOfText2[9]  := LineOfText2[8]
  LineOfText2[8]  := "."

PRI Match | c1, c2, i
'Compare strings lineoftext2 = lineoftext3 length 12.
'String lineoftext3 may contain '?' wildcards
'Returns true is they match, else false
  i~
  repeat 12
    c1 := LineOfText2[i]                                ' filename
    c2 := LineOfText3[i++]                              ' wildname
    if "a" =< c1 and c1 =< "z"
      c1 += "A" - "a"
    if "a" =< c2 and c2 =< "z"
      c2 += "A" - "a"
    if c2 == "?"
      next
    if c1 <> c2
      return false
  return true    


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}} 